@@ -1,15 +1,35 @@
-from airflow_commons.utils.bigquery_utils import *
+from google.cloud import bigquery
+
+from airflow_commons.glossary import COMMA
 from airflow_commons.logger import LOGGER
+from airflow_commons.sql_resources import (
+    GET_OLDEST_PARTITION_FIELD_SQL_FILE,
+    ARCHIVE_SOURCE_STATEMENT_FILE,
+    ARCHIVE_DELETE_WHERE_STATEMENT_FILE,
+)
+from airflow_commons.utils.bigquery_utils import (
+    DEFAULT_TIMEOUT,
+    DEFAULT_LOCATION,
+    connect,
+    get_time_partition_field,
+    single_value_select,
+    query,
+    get_table_ref,
+)
+from airflow_commons.utils.file_utils import read_sql
 from airflow_commons.utils.sql_utils import (
     get_deduplication_source_statement,
     get_merge_sql,
+    get_oldest_partition_field_sql,
+    get_delete_sql,
 )
-from airflow_commons.utils.file_utils import read_sql
 from airflow_commons.utils.time_utils import get_buffered_timestamp
 
-WRITE_DISPOSITIONS = ["WRITE_APPEND", "WRITE_TRUNCATE", "WRITE_EMPTY"]
 
+ARCHIVE_MODES = ["INSERT", "UPSERT"]
+WRITE_DISPOSITIONS = ["WRITE_APPEND", "WRITE_TRUNCATE", "WRITE_EMPTY"]
 DEFAULT_PRIMARY_KEYS = ["id"]
+ARCHIVE_DEFAULT_PRIMARY_KEYS = ["id", "last_updated_at", "processed_at"]
 DEFAULT_TIME_COLUMNS = ["last_updated_at", "processed_at"]
 
 
@@ -56,14 +76,21 @@ def deduplicate(
     target_partition_field = get_time_partition_field(
         client, target_dataset, target_table
     )
-    oldest_target_partition = get_oldest_partition_field(
-        client,
-        source_dataset,
-        source_table,
-        target_partition_field,
-        source_partition_field,
-        buffered_start_date,
-        end_date,
+    oldest_target_partition = single_value_select(
+        client=client,
+        sql=get_oldest_partition_field_sql(
+            sql_file=GET_OLDEST_PARTITION_FIELD_SQL_FILE,
+            project_id=project_id,
+            target_dataset=target_dataset,
+            target_table=target_table,
+            source_dataset=source_dataset,
+            source_table=source_table,
+            target_partition_field=target_partition_field,
+            source_partition_field=source_partition_field,
+            start_date=buffered_start_date,
+            end_date=end_date,
+            primary_keys=primary_keys,
+        ),
     )
     if oldest_target_partition:
         source_statement = get_deduplication_source_statement(
@@ -89,7 +116,7 @@ def deduplicate(
             target_partition_field=target_partition_field,
             oldest_target_partition=oldest_target_partition,
         )
-        query(
+        return query(
             client=client,
             job_config=bigquery.QueryJobConfig(),
             sql=sql,
@@ -106,6 +133,7 @@ def deduplicate(
                 end_date=end_date,
             )
         )
+        return None
 
 
 def upsert(
@@ -150,7 +178,7 @@ def upsert(
         primary_keys=primary_keys,
         mode="UPSERT",
     )
-    query(
+    return query(
         client=client,
         job_config=bigquery.QueryJobConfig(),
         sql=sql,
@@ -201,7 +229,7 @@ def insert(
         primary_keys=primary_keys,
         mode="INSERT",
     )
-    query(
+    return query(
         client=client,
         job_config=bigquery.QueryJobConfig(),
         sql=sql,
@@ -211,7 +239,7 @@ def insert(
 
 
 def write_query_results_to_destination(
-    credentials_file: str,
+    service_account_file: str,
     sql_file: str,
     timeout: int,
     location: str,
@@ -221,7 +249,7 @@ def write_query_results_to_destination(
 ):
     """
 
-    :param credentials_file: relative location of service account file
+    :param service_account_file: relative location of service account file
     :param sql_file: relative location of sql file to read sql from
     :param timeout: job timeout parameter
     :param location: job location parameter
@@ -236,7 +264,7 @@ def write_query_results_to_destination(
                 write_disposition
             )
         )
-    client = connect(credentials_file)
+    client = connect(service_account_file)
     job_config = bigquery.QueryJobConfig()
     job_config.write_disposition = write_disposition
     job_config.allow_large_results = True
@@ -244,11 +272,11 @@ def write_query_results_to_destination(
         client, destination_dataset, destination_table
     )
     sql = read_sql(sql_file=sql_file)
-    query(client, job_config, sql, timeout, location)
+    return query(client, job_config, sql, timeout, location)
 
 
 def get_query_results(
-    credentials_file: str,
+    service_account_file: str,
     sql_file: str,
     sql_params: dict = None,
     timeout=DEFAULT_TIMEOUT,
@@ -257,14 +285,14 @@ def get_query_results(
     """
     Runs query and returns the result as dataframe
 
-    :param credentials_file: relative location of service account file
+    :param service_account_file: relative location of service account file
     :param sql_file: relative location of sql file to read sql from
     :param sql_params: sql parameters dictionary
     :param timeout: query timeout duration in seconds
     :param location: query location parameter
     :return: query result
     """
-    client = connect(credentials_file=credentials_file)
+    client = connect(service_account_file=service_account_file)
     if sql_params is None:
         sql_params = dict()
     sql = read_sql(sql_file=sql_file, **sql_params)
@@ -278,7 +306,7 @@ def get_query_results(
 
 
 def get_single_value_query_results(
-    credentials_file: str,
+    service_account_file: str,
     sql_file: str,
     sql_params: dict = None,
     timeout=DEFAULT_TIMEOUT,
@@ -287,14 +315,14 @@ def get_single_value_query_results(
     """
     Runs a single value returning query and returns the result
 
-    :param credentials_file: relative location of service account file
+    :param service_account_file: relative location of service account file
     :param sql_file: relative location of sql file to read sql from
     :param sql_params: sql parameters dictionary
     :param timeout: query timeout duration in seconds
     :param location: query location parameter
     :return: query result
     """
-    client = connect(credentials_file=credentials_file)
+    client = connect(service_account_file=service_account_file)
     if sql_params is None:
         sql_params = dict()
     sql = read_sql(sql_file=sql_file, **sql_params)
@@ -304,3 +332,132 @@ def get_single_value_query_results(
         timeout=timeout,
         location=location,
     )
+
+
+def delete(
+    service_account_file: str,
+    project_id: str,
+    dataset_id: str,
+    table_id: str,
+    where_statement_file: str,
+    where_statement_params: dict = None,
+    timeout=DEFAULT_TIMEOUT,
+    location=DEFAULT_LOCATION,
+):
+    """
+    Runs a delete query on given table, and removes rows that conform where condition
+
+    :param service_account_file: relative location of service account file
+    :param project_id: Bigquery project id
+    :param dataset_id: dataset id
+    :param table_id: table id
+    :param where_statement_file: relative location of where statement sql file
+    :param where_statement_params: parameters of where statements
+    :param timeout: query timeout duration in seconds
+    :param location: query location parameter
+    :return: query result
+    """
+    client = connect(service_account_file=service_account_file)
+    if where_statement_params is None:
+        where_statement_params = dict()
+    where_statement = read_sql(sql_file=where_statement_file, **where_statement_params)
+    sql = get_delete_sql(
+        project_id=project_id,
+        dataset_id=dataset_id,
+        table_id=table_id,
+        where_statement=where_statement,
+    )
+    return query(
+        client=client,
+        job_config=bigquery.QueryJobConfig(),
+        sql=sql,
+        timeout=timeout,
+        location=location,
+    )
+
+
+def archive(
+    service_account_file: str,
+    archive_date: str,
+    project_id: str,
+    source_dataset: str,
+    source_table: str,
+    target_dataset: str,
+    target_table: str,
+    primary_keys: list = None,
+    mode: str = "INSERT",
+    timeout=DEFAULT_TIMEOUT,
+    location=DEFAULT_LOCATION,
+):
+    """
+    Archives selected rows on target non-partitioned table.
+
+    :param service_account_file: relative location of service account file
+    :param archive_date: archive date
+    :param project_id: Bigquery project id
+    :param source_dataset: source dataset id
+    :param source_table: source table id
+    :param target_dataset: target dataset id
+    :param target_table: target table id
+    :param primary_keys: primary key columns of the source and target tables
+    :param mode: archive mode, INSERT only inserts non-existing rows and UPSERT updates and inserts
+    :param timeout: query timeout duration in seconds
+    :param location: query location parameter
+    :return:
+    """
+    if mode not in ARCHIVE_MODES:
+        raise ValueError(
+            "Invalid archive mode:{}. Acceptable column names are ".format(mode)
+            + COMMA.join(i for i in ARCHIVE_MODES)
+        )
+    if primary_keys is None:
+        primary_keys = ARCHIVE_DEFAULT_PRIMARY_KEYS
+    source_statement_params = dict()
+    source_statement_params["project_id"] = project_id
+    source_statement_params["source_dataset"] = source_dataset
+    source_statement_params["source_table"] = source_table
+    source_statement_params["archive_date"] = archive_date
+    source_statement_params["source_partition_field"] = get_time_partition_field(
+        client=connect(service_account_file=service_account_file),
+        dataset_id=source_dataset,
+        table_id=source_table,
+    )
+    if mode == "INSERT":
+        insert(
+            service_account_file=service_account_file,
+            source_statement_file=ARCHIVE_SOURCE_STATEMENT_FILE,
+            project_id=project_id,
+            target_dataset=target_dataset,
+            target_table=target_table,
+            source_statement_params=source_statement_params,
+            primary_keys=primary_keys,
+            timeout=timeout,
+            location=location,
+        )
+    elif mode == "UPSERT":
+        upsert(
+            service_account_file=service_account_file,
+            source_statement_file=ARCHIVE_SOURCE_STATEMENT_FILE,
+            project_id=project_id,
+            target_dataset=target_dataset,
+            target_table=target_table,
+            source_statement_params=source_statement_params,
+            primary_keys=primary_keys,
+            timeout=timeout,
+            location=location,
+        )
+    delete_where_statement_params = dict()
+    delete_where_statement_params["archive_date"] = archive_date
+    delete_where_statement_params["source_partition_field"] = source_statement_params[
+        "source_partition_field"
+    ]
+    delete(
+        service_account_file=service_account_file,
+        project_id=project_id,
+        dataset_id=source_dataset,
+        table_id=source_table,
+        where_statement_file=ARCHIVE_DELETE_WHERE_STATEMENT_FILE,
+        where_statement_params=delete_where_statement_params,
+        timeout=timeout,
+        location=location,
+    )
